@@ -25,8 +25,15 @@ from collections import defaultdict
 from importlib import import_module
 from inspect import cleandoc
 import json
-import re
 import logging
+import re
+import sys
+
+try:
+    from pkg_resources import get_distribution
+except ImportError:
+    # Setuptools might not be available.
+    get_distribution = None
 
 from aiohttp import web
 
@@ -38,6 +45,11 @@ from immp.hook.web import WebHook
 log = logging.getLogger(__name__)
 
 
+def _module_doc(obj):
+    doc = import_module(obj.__module__).__doc__
+    return cleandoc(doc) if doc else None
+
+
 class WebUIHook(immp.ResourceHook):
     """
     Hook providing web-based configuration management for a running host instance.
@@ -48,6 +60,8 @@ class WebUIHook(immp.ResourceHook):
     def __init__(self, name, config, host):
         super().__init__(name, config, host)
         self.ctx = None
+        self._host_version = get_distribution(immp.__name__).version if get_distribution else None
+        self._python_version = ".".join(str(v) for v in sys.version_info[:3])
 
     def on_load(self):
         log.debug("Registering routes")
@@ -114,7 +128,8 @@ class WebUIHook(immp.ResourceHook):
                    [(module, logging.getLevelName(logger.level))
                     for module, logger in sorted(logging.root.manager.loggerDict.items())
                     if isinstance(logger, logging.Logger) and logger.level != logging.NOTSET])
-        return {"loggers": loggers}
+        return {"loggers": loggers,
+                "versions": [("Python", self._python_version), ("IMMP", self._host_version)]}
 
     async def add(self, request):
         post = await request.post()
@@ -133,7 +148,7 @@ class WebUIHook(immp.ResourceHook):
             return {"path": path,
                     "config": config,
                     "class": cls,
-                    "doc": cleandoc(import_module(cls.__module__).__doc__),
+                    "doc": _module_doc(cls),
                     "hook": issubclass(cls, immp.Hook)}
         try:
             name = post["name"]
@@ -184,7 +199,7 @@ class WebUIHook(immp.ResourceHook):
             title = await immp.Channel(plug, source).title()
             name = re.sub(r"[^a-z0-9]+", "-", title, flags=re.I).strip("-") if title else ""
         return {"plug": plug,
-                "doc": cleandoc(import_module(plug.__module__).__doc__),
+                "doc": _module_doc(plug.__class__),
                 "add_name": name,
                 "add_source": source,
                 "channels": {name: channel for name, channel in self.host.channels.items()
@@ -436,7 +451,7 @@ class WebUIHook(immp.ResourceHook):
         hook = self._resolve_hook(request)
         can_stop = not isinstance(hook, (WebHook, WebUIHook))
         return {"hook": hook,
-                "doc": cleandoc(import_module(hook.__module__).__doc__),
+                "doc": _module_doc(hook),
                 "resource": isinstance(hook, immp.ResourceHook),
                 "priority": self.host.priority.get(hook.name),
                 "can_stop": can_stop}
